@@ -81,19 +81,24 @@ class PropertyRule:
             # Value is already evaluated
             assign_type = type(value)
 
-        # Check if value's type is compatible
+        # First check if the value's type is already directly compatible
         for valid_type in self.valid_types:
-            if expressions.is_castable(assign_type, valid_type):
-                if isinstance(value, expressions.Expr):
-                    # Found a type-compatible match. (first match is best match)
-                    # Wrap the expression with an explicit assignment cast
-                    value = expressions.AssignmentCast(self.env, src_ref, value, valid_type)
+            if assign_type == valid_type:
                 break
         else:
-            self.env.msg.fatal(
-                "Incompatible assignment to property '%s'" % self.get_name(),
-                src_ref
-            )
+            # otherwise, cast to the first compatible type
+            for valid_type in self.valid_types:
+                if expressions.is_castable(assign_type, valid_type):
+                    if isinstance(value, expressions.Expr):
+                        # Found a type-compatible match. (first match is best match)
+                        # Wrap the expression with an explicit assignment cast
+                        value = expressions.AssignmentCast(self.env, src_ref, value, valid_type)
+                    break
+            else:
+                self.env.msg.fatal(
+                    "Incompatible assignment to property '%s'" % self.get_name(),
+                    src_ref
+                )
 
         # Store the property
         comp_def.properties[self.get_name()] = value
@@ -226,6 +231,16 @@ class Prop_dontcompare(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "O"
 
+    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
+        super().assign_value(comp_def, value, src_ref)
+
+        # int type only makes sense if assigned to a field (since it is a bitmask)
+        # If assigned to any other components, exclusively cast it to a boolean
+        if not isinstance(comp_def, comp.Field):
+            value = comp_def.properties[self.get_name()]
+            value = expressions.AssignmentCast(self.env, src_ref, value, bool)
+            comp_def.properties[self.get_name()] = value
+
     def validate(self, node: m_node.Node, value: Any) -> None:
         donttest = node.get_property("donttest")
 
@@ -298,6 +313,16 @@ class Prop_donttest(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "O"
 
+    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
+        super().assign_value(comp_def, value, src_ref)
+
+        # int type only makes sense if assigned to a field (since it is a bitmask)
+        # If assigned to any other components, exclusively cast it to a boolean
+        if not isinstance(comp_def, comp.Field):
+            value = comp_def.properties[self.get_name()]
+            value = expressions.AssignmentCast(self.env, src_ref, value, bool)
+            comp_def.properties[self.get_name()] = value
+
     def validate(self, node: m_node.Node, value: Any) -> None:
         if isinstance(node, m_node.FieldNode):
             # 5.2.2.1-a: If value is a bit mask, the mask shall have the same width
@@ -963,21 +988,29 @@ class Prop_singlepulse(PropertyRule):
 
 class Prop_we(PropertyRule):
     bindable_to = {comp.Field}
-    valid_types = (bool,)
+    valid_types = (bool, comp.Signal, comp.Field)
     default = False
     dyn_assign_allowed = True
     mutex_group = "C"
 
     def validate(self, node: m_node.Node, value: Any) -> None:
         assert isinstance(node, m_node.FieldNode)
-        if value and (node.get_property('hw') not in (rdltypes.AccessType.rw, rdltypes.AccessType.w)):
+        self._validate_ref_width_is_1(node, "we", value)
+
+        if isinstance(value, comp.VectorComponent):
+            uses_we = True
+        else:
+            # value is boolean
+            uses_we = value
+
+        if uses_we and (node.get_property('hw') not in (rdltypes.AccessType.rw, rdltypes.AccessType.w)):
             self.env.msg.error(
-                "Property 'we' is 'true' on field '%s', but the field is not writable by hardware"
+                "Field '%s' sets property 'we', but the field's 'hw' property indicates is not writable by hardware"
                 % (node.inst_name),
                 node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
             )
 
-        if value and not node.implements_storage:
+        if uses_we and not node.implements_storage:
             self.env.msg.error(
                 "Use of 'we' property on field '%s' that does not implement storage does not make sense"
                 % (node.inst_name),
@@ -986,21 +1019,29 @@ class Prop_we(PropertyRule):
 
 class Prop_wel(PropertyRule):
     bindable_to = {comp.Field}
-    valid_types = (bool,)
+    valid_types = (bool, comp.Signal, comp.Field)
     default = False
     dyn_assign_allowed = True
     mutex_group = "C"
 
     def validate(self, node: m_node.Node, value: Any) -> None:
         assert isinstance(node, m_node.FieldNode)
-        if value and (node.get_property('hw') not in (rdltypes.AccessType.rw, rdltypes.AccessType.w)):
+        self._validate_ref_width_is_1(node, "we", value)
+
+        if isinstance(value, comp.VectorComponent):
+            uses_we = True
+        else:
+            # value is boolean
+            uses_we = value
+
+        if uses_we and (node.get_property('hw') not in (rdltypes.AccessType.rw, rdltypes.AccessType.w)):
             self.env.msg.error(
-                "Property 'wel' is 'true' on field '%s', but the field is not writable by hardware"
+                "Field '%s' sets property 'wel', but the field's 'hw' property indicates is not writable by hardware"
                 % (node.inst_name),
                 node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
             )
 
-        if value and not node.implements_storage:
+        if uses_we and not node.implements_storage:
             self.env.msg.error(
                 "Use of 'wel' property on field '%s' that does not implement storage does not make sense"
                 % (node.inst_name),
@@ -1047,18 +1088,24 @@ class Prop_fieldwidth(PropertyRule):
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Prop_hwclr(PropertyRule):
     bindable_to = {comp.Field}
-    valid_types = (bool,)
+    valid_types = (bool, comp.Signal, comp.Field)
     default = False
     dyn_assign_allowed = True
     mutex_group = None
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        self._validate_ref_width_is_1(node, "hwclr", value)
+
 class Prop_hwset(PropertyRule):
     bindable_to = {comp.Field}
-    valid_types = (bool,)
+    valid_types = (bool, comp.Signal, comp.Field)
     default = False
     dyn_assign_allowed = True
     mutex_group = None
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        self._validate_ref_width_is_1(node, "hwset", value)
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Prop_hwenable(PropertyRule):
     bindable_to = {comp.Field}
